@@ -45,6 +45,7 @@ struct _GIRepositoryPrivate
   GHashTable *typelibs; /* (string) namespace -> GITypelib */
   GHashTable *lazy_typelibs; /* (string) namespace-version -> GITypelib */
   GHashTable *info_by_gtype; /* GType -> GIBaseInfo */
+  GHashTable *info_by_error_domain; /* GQuark -> GIBaseInfo */
 };
 
 G_DEFINE_TYPE (GIRepository, g_irepository, G_TYPE_OBJECT);
@@ -64,6 +65,10 @@ g_irepository_init (GIRepository *repository)
     = g_hash_table_new_full (g_direct_hash, g_direct_equal,
                              (GDestroyNotify) NULL,
                              (GDestroyNotify) g_base_info_unref);
+  repository->priv->info_by_error_domain
+    = g_hash_table_new_full (g_direct_hash, g_direct_equal,
+                             (GDestroyNotify) NULL,
+                             (GDestroyNotify) g_base_info_unref);
 }
 
 static void
@@ -74,6 +79,7 @@ g_irepository_finalize (GObject *object)
   g_hash_table_destroy (repository->priv->typelibs);
   g_hash_table_destroy (repository->priv->lazy_typelibs);
   g_hash_table_destroy (repository->priv->info_by_gtype);
+  g_hash_table_destroy (repository->priv->info_by_error_domain);
 
   (* G_OBJECT_CLASS (g_irepository_parent_class)->finalize) (G_OBJECT (repository));
 }
@@ -374,7 +380,7 @@ register_internal (GIRepository *repository,
 
 /**
  * g_irepository_get_dependencies:
- * @repository: A #GIRepository, may be %NULL for the default
+ * @repository: (allow-none): A #GIRepository, may be %NULL for the default
  * @namespace_: Namespace of interest
  *
  * Return an array of all (transitive) dependencies for namespace
@@ -382,7 +388,7 @@ register_internal (GIRepository *repository,
  * form <code>namespace-version</code>.
  *
  * Note: The namespace must have already been loaded using a function
- * such as #g_irepository_require before calling this function.
+ * such as g_irepository_require() before calling this function.
  *
  * Returns: (transfer full): Zero-terminated string array of versioned dependencies
  */
@@ -440,14 +446,14 @@ g_irepository_load_typelib (GIRepository *repository,
 
 /**
  * g_irepository_is_registered:
- * @repository: A #GIRepository, may be %NULL for the default
+ * @repository: (allow-none): A #GIRepository, may be %NULL for the default
  * @namespace_: Namespace of interest
  * @version: (allow-none): Required version, may be %NULL for latest
  *
  * Check whether a particular namespace (and optionally, a specific
  * version thereof) is currently loaded.  This function is likely to
  * only be useful in unusual circumstances; in order to act upon
- * metadata in the namespace, you should call #g_irepository_require
+ * metadata in the namespace, you should call g_irepository_require()
  * instead which will ensure the namespace is loaded, and return as
  * quickly as this function will if it has already been loaded.
  *
@@ -486,7 +492,7 @@ g_irepository_get_default (void)
 
 /**
  * g_irepository_get_n_infos:
- * @repository: A #GIRepository, may be %NULL for the default
+ * @repository: (allow-none): A #GIRepository, may be %NULL for the default
  * @namespace_: Namespace to inspect
  *
  * This function returns the number of metadata entries in
@@ -515,117 +521,17 @@ g_irepository_get_n_infos (GIRepository *repository,
   return n_interfaces;
 }
 
-typedef struct
-{
-  GIRepository *repo;
-  gint index;
-  const gchar *name;
-  gboolean type_firstpass;
-  const gchar *type;
-  GIBaseInfo *iface;
-} IfaceData;
-
-static void
-find_interface (gpointer key,
-		gpointer value,
-		gpointer data)
-{
-  gint i;
-  GITypelib *typelib = (GITypelib *)value;
-  Header *header = (Header *) typelib->data;
-  IfaceData *iface_data = (IfaceData *)data;
-  gint index;
-  gint n_entries;
-  const gchar *name;
-  const gchar *type;
-  DirEntry *entry;
-
-  index = 0;
-  n_entries = ((Header *)typelib->data)->n_local_entries;
-
-  if (iface_data->name)
-    {
-      for (i = 1; i <= n_entries; i++)
-	{
-	  entry = g_typelib_get_dir_entry (typelib, i);
-	  name = g_typelib_get_string (typelib, entry->name);
-	  if (strcmp (name, iface_data->name) == 0)
-	    {
-	      index = i;
-	      break;
-	    }
-	}
-    }
-  else if (iface_data->type)
-    {
-      const char *c_prefix;
-      /* Inside each typelib, we include the "C prefix" which acts as
-       * a namespace mechanism.  For GtkTreeView, the C prefix is Gtk.
-       * Given the assumption that GTypes for a library also use the
-       * C prefix, we know we can skip examining a typelib if our
-       * target type does not have this typelib's C prefix.
-       *
-       * However, not every class library necessarily conforms to this,
-       * e.g. Clutter has Cogl inside it.  So, we split this into two
-       * passes.  First we try a lookup, skipping things which don't
-       * have the prefix.  If that fails then we try a global lookup,
-       * ignoring the prefix.
-       *
-       * See http://bugzilla.gnome.org/show_bug.cgi?id=564016
-       */
-      c_prefix = g_typelib_get_string (typelib, header->c_prefix);
-      if (iface_data->type_firstpass && c_prefix != NULL)
-        {
-          if (g_ascii_strncasecmp (c_prefix, iface_data->type, strlen (c_prefix)) != 0)
-            return;
-        }
-
-      for (i = 1; i <= n_entries; i++)
-	{
-	  RegisteredTypeBlob *blob;
-
-	  entry = g_typelib_get_dir_entry (typelib, i);
-	  if (!BLOB_IS_REGISTERED_TYPE (entry))
-	    continue;
-
-	  blob = (RegisteredTypeBlob *)(&typelib->data[entry->offset]);
-	  if (!blob->gtype_name)
-	    continue;
-
-	  type = g_typelib_get_string (typelib, blob->gtype_name);
-	  if (strcmp (type, iface_data->type) == 0)
-	    {
-	      index = i;
-	      break;
-	    }
-	}
-    }
-  else if (iface_data->index > n_entries)
-    iface_data->index -= n_entries;
-  else if (iface_data->index > 0)
-    {
-      index = iface_data->index;
-      iface_data->index = 0;
-    }
-
-  if (index != 0)
-    {
-      entry = g_typelib_get_dir_entry (typelib, index);
-      iface_data->iface = _g_info_new_full (entry->blob_type,
-				 	    iface_data->repo,
-					    NULL, typelib, entry->offset);
-    }
-}
-
 /**
  * g_irepository_get_info:
- * @repository: A #GIRepository, may be %NULL for the default
+ * @repository: (allow-none): A #GIRepository, may be %NULL for the default
  * @namespace_: Namespace to inspect
- * @index: Offset into namespace metadata for entry
+ * @index: 0-based offset into namespace metadata for entry
  *
  * This function returns a particular metadata entry in the
  * given namespace @namespace_.  The namespace must have
  * already been loaded before calling this function.
+ * See g_irepository_get_n_infos() to find the maximum number of
+ * entries.
  *
  * Returns: (transfer full): #GIBaseInfo containing metadata
  */
@@ -634,31 +540,53 @@ g_irepository_get_info (GIRepository *repository,
 			const gchar  *namespace,
 			gint          index)
 {
-  IfaceData data;
   GITypelib *typelib;
+  DirEntry *entry;
 
   g_return_val_if_fail (namespace != NULL, NULL);
 
   repository = get_repository (repository);
 
-  data.repo = repository;
-  data.name = NULL;
-  data.type = NULL;
-  data.index = index + 1;
-  data.iface = NULL;
-
   typelib = get_registered (repository, namespace, NULL);
 
   g_return_val_if_fail (typelib != NULL, NULL);
 
-  find_interface ((void *)namespace, typelib, &data);
+  entry = g_typelib_get_dir_entry (typelib, index + 1);
+  if (entry == NULL)
+    return NULL;
+  return _g_info_new_full (entry->blob_type,
+			   repository,
+			   NULL, typelib, entry->offset);
+}
 
-  return data.iface;
+typedef struct {
+  GIRepository *repository;
+  GType type;
+
+  gboolean fastpass;
+  GITypelib *result_typelib;
+  DirEntry *result;
+} FindByGTypeData;
+
+static void
+find_by_gtype_foreach (gpointer key,
+		       gpointer value,
+		       gpointer datap)
+{
+  GITypelib *typelib = (GITypelib*)value;
+  FindByGTypeData *data = datap;
+
+  if (data->result != NULL)
+    return;
+
+  data->result = g_typelib_get_dir_entry_by_gtype (typelib, data->fastpass, data->type);
+  if (data->result)
+    data->result_typelib = typelib;
 }
 
 /**
  * g_irepository_find_by_gtype:
- * @repository: A #GIRepository, may be %NULL for the default
+ * @repository: (allow-none): A #GIRepository, may be %NULL for the default
  * @gtype: GType to search for
  *
  * Searches all loaded namespaces for a particular #GType.  Note that
@@ -674,51 +602,59 @@ GIBaseInfo *
 g_irepository_find_by_gtype (GIRepository *repository,
 			     GType         gtype)
 {
-  IfaceData data;
+  FindByGTypeData data;
+  GIBaseInfo *cached;
 
   repository = get_repository (repository);
 
-  data.iface = g_hash_table_lookup (repository->priv->info_by_gtype,
-                                    (gpointer)gtype);
+  cached = g_hash_table_lookup (repository->priv->info_by_gtype,
+				(gpointer)gtype);
 
-  if (data.iface)
-    return g_base_info_ref (data.iface);
+  if (cached != NULL)
+    return g_base_info_ref (cached);
 
-  data.repo = repository;
-  data.name = NULL;
-  data.type_firstpass = TRUE;
-  data.type = g_type_name (gtype);
-  data.index = -1;
-  data.iface = NULL;
+  data.repository = repository;
+  data.fastpass = TRUE;
+  data.type = gtype;
+  data.result_typelib = NULL;
+  data.result = NULL;
 
-  g_hash_table_foreach (repository->priv->typelibs, find_interface, &data);
-  g_hash_table_foreach (repository->priv->lazy_typelibs, find_interface, &data);
+  g_hash_table_foreach (repository->priv->typelibs, find_by_gtype_foreach, &data);
+  if (data.result == NULL)
+    g_hash_table_foreach (repository->priv->lazy_typelibs, find_by_gtype_foreach, &data);
 
   /* We do two passes; see comment in find_interface */
-  if (!data.iface)
+  if (data.result == NULL)
     {
-      data.type_firstpass = FALSE;
-      g_hash_table_foreach (repository->priv->typelibs, find_interface, &data);
-      g_hash_table_foreach (repository->priv->lazy_typelibs, find_interface, &data);
+      data.fastpass = FALSE;
+      g_hash_table_foreach (repository->priv->typelibs, find_by_gtype_foreach, &data);
     }
+  if (data.result == NULL)
+    g_hash_table_foreach (repository->priv->lazy_typelibs, find_by_gtype_foreach, &data);
 
-  if (data.iface)
-    g_hash_table_insert (repository->priv->info_by_gtype,
-                         (gpointer) gtype,
-                         g_base_info_ref (data.iface));
+  if (data.result != NULL)
+    {
+      cached = _g_info_new_full (data.result->blob_type,
+				 repository,
+				 NULL, data.result_typelib, data.result->offset);
 
-  return data.iface;
+      g_hash_table_insert (repository->priv->info_by_gtype,
+			   (gpointer) gtype,
+			   g_base_info_ref (cached));
+      return cached;
+    }
+  return NULL;
 }
 
 /**
  * g_irepository_find_by_name:
- * @repository: A #GIRepository, may be %NULL for the default
+ * @repository: (allow-none): A #GIRepository, may be %NULL for the default
  * @namespace_: Namespace which will be searched
  * @name: Entry name to find
  *
  * Searches for a particular entry in a namespace.  Before calling
  * this function for a particular namespace, you must call
- * #g_irepository_require once to load the namespace, or otherwise
+ * g_irepository_require() once to load the namespace, or otherwise
  * ensure the namespace has already been loaded.
  *
  * Returns: (transfer full): #GIBaseInfo representing metadata about @name, or %NULL
@@ -728,26 +664,98 @@ g_irepository_find_by_name (GIRepository *repository,
 			    const gchar  *namespace,
 			    const gchar  *name)
 {
-  IfaceData data;
   GITypelib *typelib;
+  DirEntry *entry;
 
   g_return_val_if_fail (namespace != NULL, NULL);
 
   repository = get_repository (repository);
-
-  data.repo = repository;
-  data.name = name;
-  data.type = NULL;
-  data.index = -1;
-  data.iface = NULL;
-
   typelib = get_registered (repository, namespace, NULL);
-
   g_return_val_if_fail (typelib != NULL, NULL);
 
-  find_interface ((void *)namespace, typelib, &data);
+  entry = g_typelib_get_dir_entry_by_name (typelib, name);
+  if (entry == NULL)
+    return NULL;
+  return _g_info_new_full (entry->blob_type,
+			   repository,
+			   NULL, typelib, entry->offset);
+}
 
-  return data.iface;
+typedef struct {
+  GIRepository *repository;
+  GQuark domain;
+
+  GITypelib *result_typelib;
+  DirEntry *result;
+} FindByErrorDomainData;
+
+static void
+find_by_error_domain_foreach (gpointer key,
+			      gpointer value,
+			      gpointer datap)
+{
+  GITypelib *typelib = (GITypelib*)value;
+  FindByErrorDomainData *data = datap;
+
+  if (data->result != NULL)
+    return;
+
+  data->result = g_typelib_get_dir_entry_by_error_domain (typelib, data->domain);
+  if (data->result)
+    data->result_typelib = typelib;
+}
+
+/**
+ * g_irepository_find_by_error_domain:
+ * @repository: (allow-none): A #GIRepository, may be %NULL for the default
+ * @domain: a #GError domain
+ *
+ * Searches for the enum type corresponding to the given #GError
+ * domain. Before calling this function for a particular namespace,
+ * you must call g_irepository_require() once to load the namespace, or
+ * otherwise ensure the namespace has already been loaded.
+ *
+ * Returns: (transfer full): #GIEnumInfo representing metadata about @domain's
+ * enum type, or %NULL
+ *
+ * Since: 1.29.17
+ */
+GIEnumInfo *
+g_irepository_find_by_error_domain (GIRepository *repository,
+				    GQuark        domain)
+{
+  FindByErrorDomainData data;
+  GIEnumInfo *cached;
+
+  repository = get_repository (repository);
+
+  cached = g_hash_table_lookup (repository->priv->info_by_error_domain,
+				GUINT_TO_POINTER (domain));
+
+  if (cached != NULL)
+    return g_base_info_ref ((GIBaseInfo *)cached);
+
+  data.repository = repository;
+  data.domain = domain;
+  data.result_typelib = NULL;
+  data.result = NULL;
+
+  g_hash_table_foreach (repository->priv->typelibs, find_by_error_domain_foreach, &data);
+  if (data.result == NULL)
+    g_hash_table_foreach (repository->priv->lazy_typelibs, find_by_error_domain_foreach, &data);
+
+  if (data.result != NULL)
+    {
+      cached = _g_info_new_full (data.result->blob_type,
+				 repository,
+				 NULL, data.result_typelib, data.result->offset);
+
+      g_hash_table_insert (repository->priv->info_by_error_domain,
+			   GUINT_TO_POINTER (domain),
+			   g_base_info_ref (cached));
+      return cached;
+    }
+  return NULL;
 }
 
 static void
@@ -762,7 +770,7 @@ collect_namespaces (gpointer key,
 
 /**
  * g_irepository_get_loaded_namespaces:
- * @repository: A #GIRepository, may be %NULL for the default
+ * @repository: (allow-none): A #GIRepository, may be %NULL for the default
  *
  * Return the list of currently loaded namespaces.
  *
@@ -791,14 +799,14 @@ g_irepository_get_loaded_namespaces (GIRepository *repository)
 
 /**
  * g_irepository_get_version:
- * @repository: A #GIRepository, may be %NULL for the default
+ * @repository: (allow-none): A #GIRepository, may be %NULL for the default
  * @namespace_: Namespace to inspect
  *
  * This function returns the loaded version associated with the given
  * namespace @namespace_.
  *
  * Note: The namespace must have already been loaded using a function
- * such as #g_irepository_require before calling this function.
+ * such as g_irepository_require() before calling this function.
  *
  * Returns: Loaded version
  */
@@ -823,7 +831,7 @@ g_irepository_get_version (GIRepository *repository,
 
 /**
  * g_irepository_get_shared_library:
- * @repository: A #GIRepository, may be %NULL for the default
+ * @repository: (allow-none): A #GIRepository, may be %NULL for the default
  * @namespace_: Namespace to inspect
  *
  * This function returns the full path to the shared C library
@@ -832,7 +840,7 @@ g_irepository_get_version (GIRepository *repository,
  * return %NULL.
  *
  * Note: The namespace must have already been loaded using a function
- * such as #g_irepository_require before calling this function.
+ * such as g_irepository_require() before calling this function.
  *
  * Returns: Full path to shared library, or %NULL if none associated
  */
@@ -860,7 +868,7 @@ g_irepository_get_shared_library (GIRepository *repository,
 
 /**
  * g_irepository_get_c_prefix
- * @repository: A #GIRepository, may be %NULL for the default
+ * @repository: (allow-none): A #GIRepository, may be %NULL for the default
  * @namespace_: Namespace to inspect
  *
  * This function returns the "C prefix", or the C level namespace
@@ -868,7 +876,7 @@ g_irepository_get_shared_library (GIRepository *repository,
  * starts with this prefix, as well each #GType in the library.
  *
  * Note: The namespace must have already been loaded using a function
- * such as #g_irepository_require before calling this function.
+ * such as g_irepository_require() before calling this function.
  *
  * Returns: C namespace prefix, or %NULL if none associated
  */
@@ -896,7 +904,7 @@ g_irepository_get_c_prefix (GIRepository *repository,
 
 /**
  * g_irepository_get_typelib_path
- * @repository: Repository, may be %NULL for the default
+ * @repository: (allow-none): Repository, may be %NULL for the default
  * @namespace_: GI namespace to use, e.g. "Gtk"
  *
  * If namespace @namespace_ is loaded, return the full path to the
@@ -1094,13 +1102,19 @@ enumerate_namespace_versions (const gchar *namespace,
 	      last_dash = strrchr (entry, '-');
 	      version = g_strndup (last_dash+1, name_end-(last_dash+1));
 	      if (!parse_version (version, &major, &minor))
-		continue;
+		{
+		  g_free (version);
+		  continue;
+		}
 	    }
 	  else
 	    continue;
 
 	  if (g_hash_table_lookup (found_versions, version) != NULL)
-	    continue;
+	    {
+	      g_free (version);
+	      continue;
+	    }
 	  g_hash_table_insert (found_versions, version, version);
 
 	  path = g_build_filename (dirname, entry, NULL);
@@ -1337,7 +1351,7 @@ require_internal (GIRepository  *repository,
  * version @version of namespace may be specified.  If @version is
  * not specified, the latest will be used.
  *
- * Returns: (transfer full): a pointer to the #GITypelib if successful, %NULL otherwise
+ * Returns: (transfer none): a pointer to the #GITypelib if successful, %NULL otherwise
  */
 GITypelib *
 g_irepository_require (GIRepository  *repository,
@@ -1372,7 +1386,7 @@ g_irepository_require (GIRepository  *repository,
  * version @version of namespace should be specified.  If @version is
  * not specified, the latest will be used.
  *
- * Returns: (transfer full): a pointer to the #GITypelib if successful, %NULL otherwise
+ * Returns: (transfer none): a pointer to the #GITypelib if successful, %NULL otherwise
  */
 GITypelib *
 g_irepository_require_private (GIRepository  *repository,
@@ -1477,6 +1491,8 @@ g_type_tag_to_string (GITypeTag type)
       return "gfloat";
     case GI_TYPE_TAG_DOUBLE:
       return "gdouble";
+    case GI_TYPE_TAG_UNICHAR:
+      return "gunichar";
     case GI_TYPE_TAG_GTYPE:
       return "GType";
     case GI_TYPE_TAG_UTF8:
@@ -1533,8 +1549,6 @@ g_info_type_to_string (GIInfoType type)
       return "interface";
     case GI_INFO_TYPE_CONSTANT:
       return "constant";
-    case GI_INFO_TYPE_ERROR_DOMAIN:
-      return "error domain";
     case GI_INFO_TYPE_UNION:
       return "union";
     case GI_INFO_TYPE_VALUE:
