@@ -101,8 +101,6 @@ _g_ir_node_type_to_string (GIrNodeTypeId type)
       return "value";
     case G_IR_NODE_CONSTANT:
       return "constant";
-    case G_IR_NODE_ERROR_DOMAIN:
-      return "error-domain";
     case G_IR_NODE_XREF:
       return "xref";
     case G_IR_NODE_UNION:
@@ -175,10 +173,6 @@ _g_ir_node_new (GIrNodeTypeId  type,
       node = g_malloc0 (sizeof (GIrNodeConstant));
       break;
 
-    case G_IR_NODE_ERROR_DOMAIN:
-      node = g_malloc0 (sizeof (GIrNodeErrorDomain));
-      break;
-
     case G_IR_NODE_XREF:
       node = g_malloc0 (sizeof (GIrNodeXRef));
       break;
@@ -334,10 +328,15 @@ _g_ir_node_free (GIrNode *node)
 	g_free (node->name);
 	g_free (enum_->gtype_name);
 	g_free (enum_->gtype_init);
+	g_free (enum_->error_domain);
 
 	for (l = enum_->values; l; l = l->next)
 	  _g_ir_node_free ((GIrNode *)l->data);
 	g_list_free (enum_->values);
+
+	for (l = enum_->methods; l; l = l->next)
+	  _g_ir_node_free ((GIrNode *)l->data);
+	g_list_free (enum_->methods);
       }
       break;
 
@@ -379,16 +378,6 @@ _g_ir_node_free (GIrNode *node)
       }
       break;
 
-    case G_IR_NODE_ERROR_DOMAIN:
-      {
-	GIrNodeErrorDomain *domain = (GIrNodeErrorDomain *)node;
-
-	g_free (node->name);
-	g_free (domain->getquark);
-	g_free (domain->codes);
-      }
-      break;
-
     case G_IR_NODE_XREF:
       {
 	GIrNodeXRef *xref = (GIrNodeXRef *)node;
@@ -482,6 +471,8 @@ _g_ir_node_get_size (GIrNode *node)
 	size = sizeof (EnumBlob);
 	for (l = enum_->values; l; l = l->next)
 	  size += _g_ir_node_get_size ((GIrNode *)l->data);
+	for (l = enum_->methods; l; l = l->next)
+	  size += _g_ir_node_get_size ((GIrNode *)l->data);
       }
       break;
 
@@ -535,10 +526,6 @@ _g_ir_node_get_size (GIrNode *node)
       size = sizeof (ConstantBlob);
       break;
 
-    case G_IR_NODE_ERROR_DOMAIN:
-      size = sizeof (ErrorDomainBlob);
-      break;
-
     case G_IR_NODE_XREF:
       size = 0;
       break;
@@ -665,16 +652,7 @@ _g_ir_node_get_full_size_internal (GIrNode *parent,
 		  size += _g_ir_node_get_full_size_internal (node, (GIrNode *)type->parameter_type2);
 		break;
 	      case GI_TYPE_TAG_ERROR:
-		{
-		  gint n;
-
-		  if (type->errors)
-		    n = g_strv_length (type->errors);
-		  else
-		    n = 0;
-
-		  size += sizeof (ErrorTypeBlob) + 2 * (n + n % 2);
-		}
+		size += sizeof (ErrorTypeBlob);
 		break;
 	      default:
 		g_error ("Unknown type tag %d\n", type->tag);
@@ -741,9 +719,13 @@ _g_ir_node_get_full_size_internal (GIrNode *parent,
 	    size += ALIGN_VALUE (strlen (enum_->gtype_name) + 1, 4);
 	    size += ALIGN_VALUE (strlen (enum_->gtype_init) + 1, 4);
 	  }
+	if (enum_->error_domain)
+	  size += ALIGN_VALUE (strlen (enum_->error_domain) + 1, 4);
 
 	for (l = enum_->values; l; l = l->next)
 	  size += _g_ir_node_get_full_size_internal (node, (GIrNode *)l->data);
+	for (l = enum_->methods; l; l = l->next)
+	  size += _g_ir_node_get_full_size_internal (node, (GIrNode *)l->data);
       }
       break;
 
@@ -844,16 +826,6 @@ _g_ir_node_get_full_size_internal (GIrNode *parent,
       }
       break;
 
-    case G_IR_NODE_ERROR_DOMAIN:
-      {
-	GIrNodeErrorDomain *domain = (GIrNodeErrorDomain *)node;
-
-	size = sizeof (ErrorDomainBlob);
-	size += ALIGN_VALUE (strlen (node->name) + 1, 4);
-	size += ALIGN_VALUE (strlen (domain->getquark) + 1, 4);
-      }
-      break;
-
     case G_IR_NODE_XREF:
       {
 	GIrNodeXRef *xref = (GIrNodeXRef *)node;
@@ -935,7 +907,7 @@ _g_ir_node_can_have_member (GIrNode    *node)
     case G_IR_NODE_ENUM:
     case G_IR_NODE_FLAGS:
     case G_IR_NODE_CONSTANT:
-    case G_IR_NODE_ERROR_DOMAIN:
+    case G_IR_NODE_INVALID_0:
     case G_IR_NODE_PARAM:
     case G_IR_NODE_TYPE:
     case G_IR_NODE_PROPERTY:
@@ -1559,21 +1531,14 @@ _g_ir_node_build_typelib (GIrNode         *node,
 		  case GI_TYPE_TAG_ERROR:
 		    {
 		      ErrorTypeBlob *blob = (ErrorTypeBlob *)&data[*offset2];
-		      gint i;
 
 		      blob->pointer = 1;
 		      blob->reserved = 0;
 		      blob->tag = type->tag;
 		      blob->reserved2 = 0;
-		      if (type->errors)
-			blob->n_domains = g_strv_length (type->errors);
-		      else
-			blob->n_domains = 0;
+		      blob->n_domains = 0;
 
-		      *offset2 = ALIGN_VALUE (*offset2 + G_STRUCT_OFFSET (ErrorTypeBlob, domains)
-		                              + 2 * blob->n_domains, 4);
-		      for (i = 0; i < blob->n_domains; i++)
-			blob->domains[i] = find_entry (build, type->errors[i]);
+		      *offset2 += sizeof (ErrorTypeBlob);
 		    }
 		    break;
 
@@ -2067,9 +2032,13 @@ _g_ir_node_build_typelib (GIrNode         *node,
 	    blob->gtype_name = 0;
 	    blob->gtype_init = 0;
 	  }
+	if (enum_->error_domain)
+	  blob->error_domain = _g_ir_write_string (enum_->error_domain, strings, data, offset2);
+	else
+	  blob->error_domain = 0;
 
 	blob->n_values = 0;
-	blob->reserved2 = 0;
+	blob->n_methods = 0;
 
 	for (l = enum_->values; l; l = l->next)
 	  {
@@ -2078,6 +2047,14 @@ _g_ir_node_build_typelib (GIrNode         *node,
 	    blob->n_values++;
 	    _g_ir_node_build_typelib (value, node, build, offset, offset2);
 	  }
+
+	for (l = enum_->methods; l; l = l->next)
+	  {
+	    GIrNode *method = (GIrNode *)l->data;
+
+	    blob->n_methods++;
+	    _g_ir_node_build_typelib (method, node, build, offset, offset2);
+	  }
       }
       break;
 
@@ -2234,22 +2211,6 @@ _g_ir_node_build_typelib (GIrNode         *node,
       }
       break;
 
-    case G_IR_NODE_ERROR_DOMAIN:
-      {
-	GIrNodeErrorDomain *domain = (GIrNodeErrorDomain *)node;
-	ErrorDomainBlob *blob = (ErrorDomainBlob *)&data[*offset];
-	*offset += sizeof (ErrorDomainBlob);
-
-	blob->blob_type = BLOB_TYPE_ERROR_DOMAIN;
-	blob->deprecated = domain->deprecated;
-	blob->reserved = 0;
-	blob->name = _g_ir_write_string (node->name, strings, data, offset2);
-	blob->get_quark = _g_ir_write_string (domain->getquark, strings, data, offset2);
-	blob->error_codes = find_entry (build, domain->codes);
-	blob->reserved2 = 0;
-      }
-      break;
-
     case G_IR_NODE_CONSTANT:
       {
 	GIrNodeConstant *constant = (GIrNodeConstant *)node;
