@@ -208,6 +208,17 @@ g_typelib_get_dir_entry_by_gtype (GITypelib *typelib,
   guint i;
   const char *c_prefix;
 
+  /* There is a corner case regarding GdkRectangle.  GdkRectangle is a
+     boxed type, but it is just an alias to boxed struct
+     CairoRectangleInt.  Scanner automatically converts all references
+     to GdkRectangle to CairoRectangleInt, so GdkRectangle does not
+     appear in the typelibs at all, although user code might query it.
+     So if we get such query, we also change it to lookup of
+     CairoRectangleInt.
+     https://bugzilla.gnome.org/show_bug.cgi?id=655423 */
+  if (!fastpass && !strcmp (gtype_name, "GdkRectangle"))
+    gtype_name = "CairoRectangleInt";
+
   /* Inside each typelib, we include the "C prefix" which acts as
    * a namespace mechanism.  For GtkTreeView, the C prefix is Gtk.
    * Given the assumption that GTypes for a library also use the
@@ -249,6 +260,36 @@ g_typelib_get_dir_entry_by_gtype (GITypelib *typelib,
   return NULL;
 }
 
+DirEntry *
+g_typelib_get_dir_entry_by_error_domain (GITypelib *typelib,
+					 GQuark     error_domain)
+{
+  Header *header = (Header *)typelib->data;
+  guint n_entries = header->n_local_entries;
+  const char *domain_string = g_quark_to_string (error_domain);
+  DirEntry *entry;
+  guint i;
+
+  for (i = 1; i <= n_entries; i++)
+    {
+      EnumBlob *blob;
+      const char *enum_domain_string;
+
+      entry = g_typelib_get_dir_entry (typelib, i);
+      if (entry->blob_type != BLOB_TYPE_ENUM)
+	continue;
+
+      blob = (EnumBlob *)(&typelib->data[entry->offset]);
+      if (!blob->error_domain)
+	continue;
+
+      enum_domain_string = g_typelib_get_string (typelib, blob->error_domain);
+      if (strcmp (domain_string, enum_domain_string) == 0)
+	return entry;
+    }
+  return NULL;
+}
+
 void
 g_typelib_check_sanity (void)
 {
@@ -284,7 +325,6 @@ g_typelib_check_sanity (void)
   CHECK_SIZE (ArrayTypeBlob, 8);
   CHECK_SIZE (ParamTypeBlob, 4);
   CHECK_SIZE (ErrorTypeBlob, 4);
-  CHECK_SIZE (ErrorDomainBlob, 16);
   CHECK_SIZE (ValueBlob, 12);
   CHECK_SIZE (FieldBlob, 16);
   CHECK_SIZE (RegisteredTypeBlob, 16);
@@ -446,7 +486,6 @@ validate_header_basic (const guint8   *memory,
       header->field_blob_size != sizeof (FieldBlob) ||
       header->value_blob_size != sizeof (ValueBlob) ||
       header->constant_blob_size != sizeof (ConstantBlob) ||
-      header->error_domain_blob_size != sizeof (ErrorDomainBlob) ||
       header->attribute_blob_size != sizeof (AttributeBlob) ||
       header->signature_blob_size != sizeof (SignatureBlob) ||
       header->enum_blob_size != sizeof (EnumBlob) ||
@@ -607,8 +646,6 @@ validate_error_type_blob (GITypelib     *typelib,
 {
   ErrorTypeBlob *blob;
   Header *header;
-  gint i;
-  DirEntry *entry;
 
   blob = (ErrorTypeBlob*)&typelib->data[offset];
 
@@ -623,30 +660,6 @@ validate_error_type_blob (GITypelib     *typelib,
       return FALSE;
     }
 
-  for (i = 0; i < blob->n_domains; i++)
-    {
-      if (blob->domains[i] == 0 || blob->domains[i] > header->n_entries)
-	{
-	  g_set_error (error,
-		       G_TYPELIB_ERROR,
-		       G_TYPELIB_ERROR_INVALID_BLOB,
-		       "Invalid directory index %d", blob->domains[i]);
-	  return FALSE;
-	}
-
-      entry = g_typelib_get_dir_entry (typelib, blob->domains[i]);
-
-      if (entry->blob_type != BLOB_TYPE_ERROR_DOMAIN &&
-	  (entry->local || entry->blob_type != BLOB_TYPE_INVALID))
-	{
-	  g_set_error (error,
-		       G_TYPELIB_ERROR,
-		       G_TYPELIB_ERROR_INVALID_BLOB,
-		       "Wrong blob type");
-	  return FALSE;
-	}
-    }
-
   return TRUE;
 }
 
@@ -1426,6 +1439,7 @@ validate_enum_blob (ValidateContext *ctx,
   GITypelib *typelib = ctx->typelib;
   EnumBlob *blob;
   gint i;
+  guint32 offset2;
 
   if (typelib->len < offset + sizeof (EnumBlob))
     {
@@ -1471,7 +1485,8 @@ validate_enum_blob (ValidateContext *ctx,
     return FALSE;
 
   if (typelib->len < offset + sizeof (EnumBlob) +
-      blob->n_values * sizeof (ValueBlob))
+      blob->n_values * sizeof (ValueBlob) +
+      blob->n_methods * sizeof (FunctionBlob))
     {
       g_set_error (error,
 		   G_TYPELIB_ERROR,
@@ -1480,22 +1495,22 @@ validate_enum_blob (ValidateContext *ctx,
       return FALSE;
     }
 
+  offset2 = offset + sizeof (EnumBlob);
+
   push_context (ctx, get_string_nofail (typelib, blob->name));
 
-  for (i = 0; i < blob->n_values; i++)
+  for (i = 0; i < blob->n_values; i++, offset2 += sizeof (ValueBlob))
     {
       if (!validate_value_blob (typelib,
-				offset + sizeof (EnumBlob) +
-				i * sizeof (ValueBlob),
+				offset2,
 				error))
 	return FALSE;
 
 #if 0
-      v1 = (ValueBlob *)&typelib->data[offset + sizeof (EnumBlob) +
-                                        i * sizeof (ValueBlob)];
+      v1 = (ValueBlob *)&typelib->data[offset2];
       for (j = 0; j < i; j++)
 	{
-	  v2 = (ValueBlob *)&typelib->data[offset + sizeof (EnumBlob) +
+	  v2 = (ValueBlob *)&typelib->data[offset2 +
                                             j * sizeof (ValueBlob)];
 
 	  if (v1->value == v2->value)
@@ -1512,6 +1527,12 @@ validate_enum_blob (ValidateContext *ctx,
 #endif
     }
 
+  for (i = 0; i < blob->n_methods; i++, offset2 += sizeof (FunctionBlob))
+    {
+      if (!validate_function_blob (ctx, offset2, BLOB_TYPE_ENUM, error))
+	return FALSE;
+    }
+
   pop_context (ctx);
 
   return TRUE;
@@ -1824,14 +1845,6 @@ validate_interface_blob (ValidateContext *ctx,
   return TRUE;
 }
 
-static gboolean
-validate_errordomain_blob (GITypelib     *typelib,
-			   guint32        offset,
-			   GError       **error)
-{
-  return TRUE;
-}
-
 static gboolean
 validate_union_blob (GITypelib     *typelib,
 		     guint32        offset,
@@ -1891,10 +1904,6 @@ validate_blob (ValidateContext *ctx,
       if (!validate_constant_blob (typelib, offset, error))
 	return FALSE;
       break;
-    case BLOB_TYPE_ERROR_DOMAIN:
-      if (!validate_errordomain_blob (typelib, offset, error))
-	return FALSE;
-      break;
     case BLOB_TYPE_UNION:
       if (!validate_union_blob (typelib, offset, error))
 	return FALSE;
