@@ -25,9 +25,10 @@ import re
 from . import message
 from .odict import odict
 
-# Tags - annotations applyed to comment blocks
+# Tags - annotations applied to comment blocks
 TAG_VFUNC = 'virtual'
 TAG_SINCE = 'since'
+TAG_STABILITY = 'stability'
 TAG_DEPRECATED = 'deprecated'
 TAG_RETURNS = 'returns'
 TAG_ATTRIBUTES = 'attributes'
@@ -37,6 +38,20 @@ TAG_UNREF_FUNC = 'unref func'
 TAG_REF_FUNC = 'ref func'
 TAG_SET_VALUE_FUNC = 'set value func'
 TAG_GET_VALUE_FUNC = 'get value func'
+TAG_TRANSFER = 'transfer'
+_ALL_TAGS = [TAG_VFUNC,
+             TAG_SINCE,
+             TAG_STABILITY,
+             TAG_DEPRECATED,
+             TAG_RETURNS,
+             TAG_ATTRIBUTES,
+             TAG_RENAME_TO,
+             TAG_TYPE,
+             TAG_UNREF_FUNC,
+             TAG_REF_FUNC,
+             TAG_SET_VALUE_FUNC,
+             TAG_GET_VALUE_FUNC,
+             TAG_TRANSFER]
 
 # Options - annotations for parameters and return values
 OPT_ALLOW_NONE = 'allow-none'
@@ -416,7 +431,6 @@ class AnnotationParser(object):
     COMMENT_HEADER_RE = re.compile(r'^\*[ \t]*\n[\t ]')
     COMMENT_HEADER_START_RE = re.compile(r'\n[\t ]')
     WHITESPACE_RE = re.compile(r'^\s*$')
-    ASCII_TEXT_RE = re.compile(r'\s*[A-Za-z]+')
     OPTION_RE = re.compile(r'\([A-Za-z]+[^(]*\)')
     RETURNS_RE = re.compile(r'^return(s?)( value)?:', re.IGNORECASE)
 
@@ -492,20 +506,33 @@ class AnnotationParser(object):
         lineno = 2
         for line in comment[pos+1:].split('\n'):
             line = line.lstrip()
-            if (not line.startswith('*') or
-                self.WHITESPACE_RE.match(line[1:])):
+            if not line.startswith('*'):
+                lineno += 1
+                continue
+            nostar_line = line[1:]
+            is_whitespace = self.WHITESPACE_RE.match(nostar_line) is not None
+            if parsing_parameters and is_whitespace:
                 # As soon as we find a line that's just whitespace,
                 # we're done parsing the parameters.
                 parsing_parameters = False
                 lineno += 1
                 continue
+            elif is_whitespace:
+                comment_lines.append('')
+                lineno += 1
+                continue
 
-            line = line[1:].lstrip()
+            # Explicitly only accept parameters of the form "* @foo" with one space.
+            is_parameter = nostar_line.startswith(' @')
+
+            # Strip the rest of the leading whitespace for the rest of
+            # the code; may not actually be necessary, but still doing
+            # it to avoid regressions.
+            line = nostar_line.lstrip()
 
             # Look for a parameter or return value.  Both of these can
             # have parenthesized options.
             first_colonspace_index = line.find(': ')
-            is_parameter = line.startswith('@')
             is_return_value = self.RETURNS_RE.search(line)
             parse_options = True
             if ((is_parameter or is_return_value)
@@ -565,7 +592,7 @@ class AnnotationParser(object):
                 # The line is of the form "Tag: some value here", like:
                 # Since: 0.8
                 tag_name = line[:first_colonspace_index]
-                if self.ASCII_TEXT_RE.match(tag_name):
+                if tag_name.lower() in _ALL_TAGS:
                     tag_name = tag_name.lower()
                     tag = DocTag(block, tag_name)
                     tag.value = line[first_colonspace_index+2:]
@@ -573,10 +600,10 @@ class AnnotationParser(object):
                     block.tags[tag_name] = tag
                 else:
                     comment_lines.append(line)
-            elif (not is_parameter):
+            elif not parsing_parameters:
                 comment_lines.append(line)
             lineno += 1
-        block.comment = '\n'.join(comment_lines)
+        block.comment = '\n'.join(comment_lines).strip()
         block.validate()
         self._blocks[block.name] = block
 
