@@ -300,7 +300,8 @@ def create_binary(transformer, options, args):
         binary = IntrospectionBinary(args)
     else:
         binary = compile_introspection_binary(options,
-                                              gdump_parser.get_get_type_functions())
+                                              gdump_parser.get_get_type_functions(),
+                                              gdump_parser.get_error_quark_functions())
 
     shlibs = resolve_shlibs(options, binary, options.libraries)
     gdump_parser.set_introspection_binary(binary)
@@ -393,14 +394,15 @@ def scanner_main(args):
 
     ss = create_source_scanner(options, args)
 
+    ap = AnnotationParser()
+    blocks = ap.parse(ss.get_comments())
+
     # Transform the C symbols into AST nodes
+    transformer.set_annotations(blocks)
     transformer.parse(ss.get_symbols())
 
     shlibs = create_binary(transformer, options, args)
 
-    ap = AnnotationParser()
-    blocks = ap.parse(ss.get_comments())
-
     main = MainTransformer(transformer, blocks)
     main.transform()
 
