@@ -54,6 +54,7 @@ class Transformer(object):
         self._include_names = set() # string namespace
         self._includepaths = []
         self._passthrough_mode = False
+        self._annotations = {}
 
     def get_includes(self):
         return self._include_names
@@ -67,6 +68,9 @@ class Transformer(object):
     def set_passthrough_mode(self):
         self._passthrough_mode = True
 
+    def set_annotations(self, annotations):
+        self._annotations = annotations
+
     def _append_new_node(self, node):
         original = self._namespace.get(node.name)
         # Special case constants here; we allow duplication to sort-of
@@ -174,6 +178,18 @@ None."""
                          % (girname, searchdirs))
         sys.exit(1)
 
+    @classmethod
+    def parse_from_gir(cls, filename, extra_include_dirs=None):
+        self = cls(None)
+        if extra_include_dirs is not None:
+            self.set_include_paths(extra_include_dirs)
+        self.set_passthrough_mode()
+        self._parse_include(filename)
+        parser = self._cachestore.load(filename)
+        self._namespace = parser.get_namespace()
+        del self._includes[self._namespace.name]
+        return self
+
     def _parse_include(self, filename, uninstalled=False):
         parser = None
         if self._cachestore is not None:
@@ -313,7 +329,10 @@ raise ValueError."""
         elif stype == CSYMBOL_TYPE_UNION:
             return self._create_union(symbol)
         elif stype == CSYMBOL_TYPE_CONST:
-            return self._create_const(symbol)
+            # Don't parse constants which are marked (skip)
+            docblock = self._annotations.get(symbol.ident)
+            if not docblock or not 'skip' in docblock.options:
+                return self._create_const(symbol)
         # Ignore variable declarations in the header
         elif stype == CSYMBOL_TYPE_OBJECT:
             pass
