@@ -212,6 +212,38 @@ regress_test_closure_one_arg (GClosure *closure, int arg)
   return ret;
 }
 
+/**
+ * regress_test_closure_variant:
+ * @closure: GClosure which takes one GVariant and returns a GVariant
+ * @arg: (transfer none): a GVariant passed as argument to @closure
+ *
+ * Return value: (transfer full): the return value of @closure
+ */
+GVariant*
+regress_test_closure_variant (GClosure *closure, const GVariant* arg)
+{
+  GValue return_value = {0, };
+  GValue arguments[1] = {{0,} };
+  int ret;
+
+  g_value_init (&return_value, G_TYPE_VARIANT);
+
+  g_value_init (&arguments[0], G_TYPE_VARIANT);
+  g_value_set_variant (&arguments[0], arg);
+
+  g_closure_invoke (closure,
+                    &return_value,
+                    1, arguments,
+                    NULL);
+
+  ret = g_value_get_variant (&return_value);
+
+  g_value_unset (&return_value);
+  g_value_unset (&arguments[0]);
+
+  return ret;
+}
+
 /**
  * regress_test_value_arg:
  * @v: (transfer none): a GValue expected to contain an int
@@ -398,7 +430,7 @@ static const char utf8_nonconst[] = "nonconst \xe2\x99\xa5 utf8";
  * regress_test_utf8_const_return:
  * Return value: UTF-8 string
  */
-G_CONST_RETURN char *
+const char *
 regress_test_utf8_const_return (void)
 {
   /* transfer mode none */
@@ -879,7 +911,7 @@ regress_test_sequence_list()
  *
  * Return value: (element-type utf8) (transfer none):
  */
-G_CONST_RETURN GList *
+const GList *
 regress_test_glist_nothing_return (void)
 {
   return regress_test_sequence_list ();
@@ -1000,7 +1032,7 @@ regress_test_sequence_slist()
  *
  * Return value: (element-type utf8) (transfer none):
  */
-G_CONST_RETURN GSList *
+const GSList *
 regress_test_gslist_nothing_return (void)
 {
   return regress_test_sequence_slist ();
@@ -1144,7 +1176,7 @@ regress_test_table_ghash_const()
  *
  * Return value: (element-type utf8 utf8) (transfer none) (allow-none):
  */
-G_CONST_RETURN GHashTable *
+const GHashTable *
 regress_test_ghash_null_return (void)
 {
   return NULL;
@@ -1155,7 +1187,7 @@ regress_test_ghash_null_return (void)
  *
  * Return value: (element-type utf8 utf8) (transfer none):
  */
-G_CONST_RETURN GHashTable *
+const GHashTable *
 regress_test_ghash_nothing_return (void)
 {
   return regress_test_table_ghash_const ();
@@ -1284,10 +1316,34 @@ regress_test_ghash_nested_everything_return2 (void)
 
 /************************************************************************/
 
+/**
+ * regress_test_garray_container_return:
+ *
+ * Returns: (transfer container) (type GLib.PtrArray) (element-type utf8):
+ */
+GPtrArray *
+regress_test_garray_container_return (void)
+{
+  GPtrArray *array;
+
+  array = g_ptr_array_new_with_free_func (g_object_unref);
+  g_ptr_array_add (array, g_strdup ("regress"));
+
+  return array;
+}
+
+/************************************************************************/
+
 /* error? */
 
 /* enums / flags */
 
+/**
+ * NUM_REGRESS_FOO: (skip)
+ *
+ * num of elements in RegressFoo
+ */
+
 GType
 regress_test_enum_get_type (void)
 {
@@ -1759,6 +1815,9 @@ enum {
   REGRESS_TEST_OBJ_SIGNAL_SIG_NEW_WITH_ARRAY_PROP,
   REGRESS_TEST_OBJ_SIGNAL_SIG_WITH_HASH_PROP,
   REGRESS_TEST_OBJ_SIGNAL_SIG_WITH_STRV,
+  REGRESS_TEST_OBJ_SIGNAL_FIRST,
+  REGRESS_TEST_OBJ_SIGNAL_CLEANUP,
+  REGRESS_TEST_OBJ_SIGNAL_ALL,
   N_REGRESS_TEST_OBJ_SIGNALS
 };
 
@@ -1855,6 +1914,39 @@ regress_test_obj_class_init (RegressTestObjClass *klass)
 		  1,
 		  G_TYPE_STRV);
 
+  regress_test_obj_signals[REGRESS_TEST_OBJ_SIGNAL_FIRST] =
+    g_signal_new ("first",
+		  G_TYPE_FROM_CLASS (gobject_class),
+		  G_SIGNAL_RUN_FIRST,
+		  0,
+		  NULL,
+		  NULL,
+		  g_cclosure_marshal_VOID__VOID,
+		  G_TYPE_NONE,
+                  0);
+
+    regress_test_obj_signals[REGRESS_TEST_OBJ_SIGNAL_CLEANUP] =
+    g_signal_new ("cleanup",
+		  G_TYPE_FROM_CLASS (gobject_class),
+		  G_SIGNAL_RUN_CLEANUP,
+		  0,
+		  NULL,
+		  NULL,
+		  g_cclosure_marshal_VOID__VOID,
+		  G_TYPE_NONE,
+                  0);
+
+    regress_test_obj_signals[REGRESS_TEST_OBJ_SIGNAL_ALL] =
+    g_signal_new ("all",
+		  G_TYPE_FROM_CLASS (gobject_class),
+		  G_SIGNAL_RUN_FIRST | G_SIGNAL_NO_RECURSE | G_SIGNAL_DETAILED | G_SIGNAL_ACTION | G_SIGNAL_NO_HOOKS,
+		  0,
+		  NULL,
+		  NULL,
+		  g_cclosure_marshal_VOID__VOID,
+		  G_TYPE_NONE,
+                  0);
+
   gobject_class->set_property = regress_test_obj_set_property;
   gobject_class->get_property = regress_test_obj_get_property;
   gobject_class->dispose = regress_test_obj_dispose;
@@ -2276,6 +2368,21 @@ regress_test_obj_null_out (RegressTestObj **obj)
     *obj = NULL;
 }
 
+/**
+ * regress_test_array_fixed_out_objects:
+ * @objs: (out) (array fixed-size=2) (transfer full): An array of #RegressTestObj
+ */
+void
+regress_test_array_fixed_out_objects (RegressTestObj ***objs)
+{
+    RegressTestObj **values = g_new(gpointer, 2);
+
+    values[0] = regress_constructor();
+    values[1] = regress_constructor();
+
+    *objs = values;
+}
+
 typedef struct _CallbackInfo CallbackInfo;
 
 struct _CallbackInfo
@@ -3110,6 +3217,60 @@ regress_test_strv_in_gvalue (void)
   return value;
 }
 
+/**
+ * regress_test_multiline_doc_comments:
+ *
+ * This is a function.
+ *
+ * It has multiple lines in the documentation.
+ *
+ * The sky is blue.
+ *
+ * You will give me your credit card number.
+ */
+void
+regress_test_multiline_doc_comments (void)
+{
+}
+
+/**
+ * regress_test_nested_parameter:
+ * @a: An integer
+ *
+ * <informaltable>
+ *   <tgroup cols="3">
+ *     <thead>
+ *       <row>
+ *         <entry>Syntax</entry>
+ *         <entry>Explanation</entry>
+ *         <entry>Examples</entry>
+ *       </row>
+ *     </thead>
+ *     <tbody>
+ *       <row>
+ *         <entry>rgb(@r, @g, @b)</entry>
+ *         <entry>An opaque color; @r, @g, @b can be either integers between
+ *                0 and 255 or percentages</entry>
+ *         <entry><literallayout>rgb(128, 10, 54)
+ * rgb(20%, 30%, 0%)</literallayout></entry>
+ *       </row>
+ *       <row>
+ *         <entry>rgba(@r, @g, @b, @a)</entry>
+ *         <entry>A translucent color; @r, @g, @b are as in the previous row,
+ *                @a is a floating point number between 0 and 1</entry>
+ *         <entry><literallayout>rgba(255, 255, 0, 0.5)</literallayout></entry>
+ *       </row>
+ *    </tbody>
+ *  </tgroup>
+ * </informaltable>
+ *
+ * What we're testing here is that the scanner ignores the @a nested inside XML.
+ */
+void
+regress_test_nested_parameter (int a)
+{
+}
+
 /**
  * regress_introspectable_via_alias:
  *
